﻿using System;
using System.Windows.Navigation;
using Microsoft.Devices.Sensors;
using Microsoft.Phone.Controls;
using Microsoft.Xna.Framework;

namespace AltitudeAndAzimuth
{
    public partial class MainPage : PhoneApplicationPage
    {
        Motion motion;

        public MainPage()
        {
            InitializeComponent();

            if (Motion.IsSupported)
            {
                motion = new Motion();
                motion.TimeBetweenUpdates = TimeSpan.FromMilliseconds(33);
                motion.CurrentValueChanged += OnMotionCurrentValueChanged;
            }
        }

        protected override void OnNavigatedTo(NavigationEventArgs args)
        {
            if (motion != null)
            {
                try { motion.Start(); }
                catch { }
            }
            base.OnNavigatedTo(args);
        }

        protected override void OnNavigatedFrom(NavigationEventArgs args)
        {
            if (motion != null)
            {
                motion.Stop();
            }
            base.OnNavigatedFrom(args);
        }

        void OnMotionCurrentValueChanged(object sender, SensorReadingEventArgs<MotionReading> args)
        {
            this.Dispatcher.BeginInvoke(() =>
            {
                Matrix matrix = args.SensorReading.Attitude.RotationMatrix;
                Matrix inverseMatrix = Matrix.Invert(matrix);
                Vector3 zAxisTransformed = Vector3.Transform(-Vector3.UnitZ, inverseMatrix);

                xText.Text = zAxisTransformed.X.ToString("F2");
                yText.Text = zAxisTransformed.Y.ToString("F2");
                zText.Text = zAxisTransformed.Z.ToString("F2");

                HorizontalCoordinate horzCoord = HorizontalCoordinate.FromMotionMatrix(args.SensorReading.Attitude.RotationMatrix);

                altitudeText.Text = horzCoord.Altitude.ToString("F0") + "°";
                azimuthText.Text = horzCoord.Azimuth.ToString("F0") + "°";
                tiltText.Text = horzCoord.Tilt.ToString("F0") + "°";
            });
        }
    }
}