﻿using System.ComponentModel;
using System.IO;
using System.Xml.Serialization;
using System.Windows.Media.Imaging;
using System.IO.IsolatedStorage;

namespace BigPicture
{
    public class PictureInfo : INotifyPropertyChanged
    {
        public const string ThumbnailsDirectory = "/thumbnails";

        bool isDownloaded, isDownloading;
        double downloadProgress;
        string downloadStatus;

        public event PropertyChangedEventHandler PropertyChanged;

        [XmlAttribute]
        public string Filename { set; get; }

        public string Url { set; get; }

        public string Title { set; get; }

        public string Artist { set; get; }

        public BitmapImage Thumbnail
        {
            get
            {
                BitmapImage bitmapImage = new BitmapImage();
                using (IsolatedStorageFile isoStorage = IsolatedStorageFile.GetUserStoreForApplication())
                {
                    string thumbnailFilename = ThumbnailsDirectory + "/" + this.Filename;

                    if (isoStorage.FileExists(thumbnailFilename))
                    {
                        using (IsolatedStorageFileStream stream = isoStorage.OpenFile(thumbnailFilename, 
                                                                                      FileMode.Open, FileAccess.Read))
                        {
                            bitmapImage.SetSource(stream);
                        }
                    }
                }
                return bitmapImage;
            }
        }

        public bool IsDownloaded
        {
            set
            {
                if (isDownloaded != value)
                {
                    isDownloaded = value;
                    OnPropertyChanged("IsDownloaded");
                    OnPropertyChanged("IsNotDownloaded");
                    OnPropertyChanged("Thumbnail");
                }
            }
            get
            {
                return isDownloaded;
            }
        }

        public bool IsNotDownloaded
        {
            get
            {
                return !isDownloaded;
            }
        }

        public bool IsDownloading
        {
            set
            {
                if (isDownloading != value)
                {
                    isDownloading = value;
                    OnPropertyChanged("IsDownloading");
                    OnPropertyChanged("IsNotDownloading");
                }
            }
            get
            {
                return isDownloading;
            }
        }

        public bool IsNotDownloading
        {
            get
            {
                return !isDownloading;
            }
        }

        public double DownloadProgress
        {
            set
            {
                if (downloadProgress != value)
                {
                    downloadProgress = value;
                    OnPropertyChanged("DownloadProgress");
                }
            }
            get
            {
                return downloadProgress;
            }
        }

        public string DownloadRequestId { set; get; }

        public string DownloadStatus
        {
            set
            {
                if (downloadStatus != value)
                {
                    downloadStatus = value;
                    OnPropertyChanged("DownloadStatus");
                }
            }
            get
            {
                return downloadStatus;
            }
        }

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
